import unittest
from jumper.vlab_hci_device import VirtualHciDevice
import subprocess


class TestHci(unittest.TestCase):
    def setUp(self):
        self.hci = VirtualHciDevice()
        self.hci.start()

    def tearDown(self):
        self.hci.stop()

    def test_sanity(self):
        hciconfig_process = subprocess.Popen(['hciconfig'], stdout=subprocess.PIPE)
        hciconfig_process.wait()
        stdout, stderr = hciconfig_process.communicate()
        self.assertRegexpMatches(stdout, 'FF:05:04:03:02:FF')



"""
:copyright: (c) 2017 by Jumper Labs Ltd.
:license: Apache 2.0, see LICENSE.txt for more details.
"""


class Interrupts(object):
    RESET = 'Reset'
    NMI = "NMI"
    HARD_FAULT = "Hard Fault"
    MEMORY_MANAGE_FAULT = "Memory Management Fault"
    BUS_FAULT = "Bus Fault"
    USAGE_FAULT = "Usage Fault"
    SVCALL = "Svcall"
    DEBUG_NON = "Debug Non"
    PENDSV = "Pendsv"
    SYSTICK = "Systick"

    # nrf52
    SPIM0_SPIS0_TWIM0_TWIS0_SPI0_TWI0 = "SPIM0_SPIS0_TWIM0_TWIS0_SPI0_TWI0"
    NFCT = "NFCT"
    CCM_AAR = "CCM_AAR"
    QDEC = "QDEC"
    COMP_LPCOMP = "COMP_LPCOMP"
    SWI0_EGU0 = "SWI0_EGU0"
    I2S = "I2S"
    FPU = "FPU"
    GPIOTE = "GPIOTE"
    UARTE0_UART0 = "UARTE0_UART0"
    TIMER0 = "TIMER0"
    TIMER1 = "TIMER1"
    TIMER2 = "TIMER2"
    TIMER3 = "TIMER3"
    TIMER4 = "TIMER4"
    RADIO = "RADIO"
    POWER_CLOCK = "POWER_CLOCK"
    RNG = "RNG"
    SPIM1_SPIS1_TWIM1_TWIS1_SPI1_TWI1 = "SPIM1_SPIS1_TWIM1_TWIS1_SPI1_TWI1"
    SWI1_EGU1 = "SWI1_EGU1"
    SWI2_EGU2 = "SWI2_EGU2"
    SWI3_EGU3 = "SWI3_EGU3"
    SWI4_EGU4 = "SWI4_EGU4"
    SWI5_EGU5 = "SWI5_EGU5"
    SPIM2_SPIS2_SPI2 = "SPIM2_SPIS2_SPI2"
    TEMP = "TEMP"
    RTC0 = "RTC0"
    RTC1 = "RTC1"
    RTC2 = "RTC2"
    WDT = "WDT"
    MWU = "MWU"
    ECB = "ECB"
    SAADC = "SAADC"
    PWM0 = "PWM0"
    PWM1 = "PWM1"
    PWM2 = "PWM2"

    # Stm32f4
    TAMP_STAMP = "TAMP_STAMP"
    EXTI0 = "EXTI0"
    EXTI1 = "EXTI1"
    EXTI2 = "EXTI2"
    EXTI3 = "EXTI3"
    EXTI4 = "EXTI4"
    EXTI9_5 = "EXTI9_5"
    EXTI15_10 = "EXTI15_10"
    ADC = "ADC"
    I2C2_EV = "I2C2_EV"
    I2C2_ER = "I2C2_ER"
    SDIO = "SDIO"
    TIM2 = "TIM2"
    TIM1_TRG_COM_TIM11 = "TIM1_TRG_COM_TIM11"
    TIM1_UP_TIM10 = "TIM1_UP_TIM10"
    TIM1_BRK_TIM9 = "TIM1_BRK_TIM9"
    TIM1_CC = "TIM1_CC"
    OTG_FS_WKUP = "OTG_FS_WKUP"
    OTG_FS = "OTG_FS"
    TIM3 = "TIM3"
    I2C1_EV = "I2C1_EV"
    I2C1_ER = "I2C1_ER"
    FLASH = "FLASH"

class JemuInterrupts(object):
    _INT = "interrupt"
    _DESCRIPTION = "description"

    def __init__(self):
        self._interrupt_callbacks = []
        self._jemu_connection = None


    def set_jemu_connection(self, jemu_connection):
        self._jemu_connection = jemu_connection
        self._jemu_connection.register(self.receive_packet)

    def on_interrupt(self, callback):
        self._interrupt_callbacks += callback

    def receive_packet(self, jemu_packet):
        if jemu_packet[self._DESCRIPTION] == self._INT:
            for callback in self._interrupt_callbacks:
                callback(jemu_packet['interrupt_type'])




"""
:copyright: (c) 2017 by Jumper Labs Ltd.
:license: Apache 2.0, see LICENSE.txt for more details.
"""
from queue import Queue
from jemu_peripheral import JemuPeripheral
from common import JemuConnectionException

class JemuMemPeripheral(JemuPeripheral):
    _LEVEL_HIGH = 1
    _LEVEL_LOW = 0

    _VALUE = "value"
    _COMMAND = "command"
    _PERIPHERAL_ID = "peripheral_id"
    _PERIPHERAL_TYPE = "peripheral_type"
    _REG_ADDRESS = "address"
    _COMMAND_SET_REG = "set_reg_value"
    _COMMAND_GET_REG = "get_reg_value"
    _DESCRIPTION = "description"
    _VALUE_RESPONSE = "value_response"

    def __init__(self, jemu_connection, id, peripheral_type, generators):
        JemuPeripheral.__init__(self, jemu_connection, id, peripheral_type, generators)

    def _set_reg_json(self, register, value):
        return {
            self._REG_ADDRESS: register,
            self._VALUE: value,
            self._COMMAND: self._COMMAND_SET_REG,
        }

    def _get_reg_json(self, register):
        return {
            self._REG_ADDRESS: register,
            self._COMMAND: self._COMMAND_GET_REG,
        }

    def set_register_value(self, register, value):
        self._jemu_connection.send_command_async(self._set_reg_json(register, value), self._id, self._peripheral_type)

    def get_register_value(self, register):
        jemu_packet = self._jemu_connection.send_command(self._get_reg_json(register), self._id, self._peripheral_type)
        if self._DESCRIPTION in jemu_packet and jemu_packet[self._DESCRIPTION] == self._VALUE_RESPONSE and self._VALUE in jemu_packet:
            return jemu_packet[self._VALUE]
        else:
            raise JemuConnectionException("Error: Couldn't get register [{}] value".format(register))



"""
:copyright: (c) 2017 by Jumper Labs Ltd.
:license: Apache 2.0, see LICENSE.txt for more details.
"""
from common import JemuConnectionException

class FieldDescriptor:
    _RESPONSE_TIMEOUT = 10

    _VALUE = "value"
    _PERIPHERAL_ID = "peripheral_id"
    _PERIPHERAL_TYPE = "peripheral_type"
    _FIELD_NAME = "field_name"
    _COMMAND = "command"
    _COMMAND_SET_VALUE = "set_value"
    _COMMAND_GET_VALUE = "get_value"
    _DESCRIPTION = "description"
    _GET_VALUE_RESPONSE = "get_value_response"

    def __init__(self, jemu_connection, id, field_name, peripheral_type):
        self._id = id
        self._jemu_connection = jemu_connection
        self._field_name = field_name
        self._peripheral_type = peripheral_type

    def set(self, val):
        self._jemu_connection.send_command_async(self._set_value_json(val), self._id, self._peripheral_type)

    def get(self):
        jemu_packet = self._jemu_connection.send_command(self._get_value_json(), self._id, self._peripheral_type)
        if self._DESCRIPTION in jemu_packet and jemu_packet[self._DESCRIPTION] == self._GET_VALUE_RESPONSE and self._VALUE in jemu_packet:
            return jemu_packet[self._VALUE]
        else:
            raise JemuConnectionException("Error: Couldn't get value from jemu peripheral")

    def _set_value_json(self, value):
        return {
            self._FIELD_NAME: self._field_name,
            self._VALUE: value,
            self._COMMAND: self._COMMAND_SET_VALUE
        }

    def _get_value_json(self):
        return {
            self._FIELD_NAME: self._field_name,
            self._COMMAND: self._COMMAND_GET_VALUE
        }


class JemuPeripheral:
    def __init__(self, jemu_connection, id, peripheral_type, generators):
        self._peripheral_type = peripheral_type
        self._id = id
        self._jemu_connection = jemu_connection

        if generators is not None:
            for field in generators:
                descriptor = FieldDescriptor(jemu_connection, id, field, peripheral_type)
                setattr(self, field, descriptor)



"""
:copyright: (c) 2017 by Jumper Labs Ltd.
:license: Apache 2.0, see LICENSE.txt for more details.
"""
from threading import Lock
from time import sleep
from common import TimeoutException
from common import WrongStateError
import sys


class JemuUartJson(object):
    _DESCRIPTION = "description"
    _VALUE_RESPONSE = "value_response"
    _COMMAND = "command"
    _WRITE_DATA = "write_uart_data"
    _READ_DATA = "read_uart_data"
    _UART_EVENT = "uart_event"
    _VALUE = "value"
    _ID = 15  # peripheral id of Uart in nRF52

    def __init__(self, vlab, mcu_name="nrf52832"):
        self._uart_callback = None
        self._jemu_socket_manager = None
        self._peripheral_type = mcu_name
        self._uart_buffer = None
        self._vlab = vlab
        self._lock = Lock()
        self._print_to_screen = False

    def print_uart_to_screen(self):
        self._print_to_screen = True

    def open(self):
        self._uart_buffer = b''
        # TODO: jemu_connection should be a member instead of using a private member from another class
        if self._vlab._jemu_connection:
            self._vlab._jemu_connection.register(self.receive_packet)

    def receive_packet(self, jemu_packet):
        if self._DESCRIPTION in jemu_packet and jemu_packet[self._DESCRIPTION] == self._UART_EVENT and self._VALUE in jemu_packet:
            with self._lock:
                new_data = chr(jemu_packet[self._VALUE])
                if self._print_to_screen:
                    # dansheme 18/4/2018: Flush is needed for testing purposes.
                    # BLE test is setting stdout of this process to PIPE and data is not being flushed automatically
                    # by the kernel.
                    sys.stdout.write(new_data)
                    sys.stdout.flush()
                self._uart_buffer += new_data

    def _write_to_uart_message(self, data):
        return {
            self._COMMAND: self._WRITE_DATA,
            self._VALUE: data
        }

    def write(self, data):
        """
         Writes data to UART

         :param data: Data to write
         """
        if self._vlab._jemu_connection:
            self._vlab._jemu_connection.send_command_async(self._write_to_uart_message(data), self._ID,
                                                          self._peripheral_type)

    def read(self):
        """
        :return: data available on the UART device. Returns an empty bytes string if no data is available.
        """
        with self._lock:
            data = self._uart_buffer
            self._uart_buffer = b''
        return data

    def wait_until_uart_receives(self, data, timeout=None):
        """
        Blocks until specified data is received on UART.
        If the device is paused, this function will continue the execution and will pause the device when the data is ready or timeout occured.
        If the device is running, the function will not pause the device.

        :param data: Expected bytes string.
        :param timeout: Emulation time specified in milliseconds. jumper.vlab.TimeoutException is raised if timeout occurred before data was available in the buffer.
        :return: The data available on in the UART buffer when the specified data was received. Note that this can be more than the data provided in the data parameter.
        """
        original_state = self._vlab.get_state()

        if original_state == 'running':
            self._vlab.pause()
        elif original_state != 'paused':
            raise WrongStateError('state should be paused or running but was {}'.format(original_state))

        if timeout is not None:
            self._vlab.stop_after_ms(timeout)

        self._vlab.resume()

        while data not in self._uart_buffer and self._vlab.get_state() == 'running':
            sleep(0.2)

        if timeout is not None:
            self._vlab.cancel_stop()

        if original_state == 'running':
            self._vlab.resume()
        elif original_state == 'paused':
            self._vlab.pause()

        with self._lock:
            if data in self._uart_buffer:
                tmp = self._uart_buffer
                self._uart_buffer = b''
                return tmp

        raise TimeoutException("timeout while waiting for data from uart")

    def read_line(self, line_separator=b'\r\n'):
        while line_separator not in self._uart_buffer:
            sleep(0.2)
        with self._lock:
            line_length = self._uart_buffer.find(line_separator) + len(line_separator)
            data = self._uart_buffer[:line_length]
            self._uart_buffer = self._uart_buffer[line_length:]
        return data



import sys, os

CORE_LINUX_OS = "linux"
CORE_WINDOWS_OS1 = "win"
CORE_WINDOWS_OS2 = "cygwin"
CORE_MAC_OS = "darwin"

JEMU_LINUX_DIR = "jemu-linux"
JEMU_MAC_DIR = "jemu-mac"
JEMU_WINDOWS_DIR = "jemu-windows"

HERE = os.path.abspath(os.path.dirname(__file__))

JEMU_DIR = None
if sys.platform.startswith(CORE_LINUX_OS):
    JEMU_DIR = os.path.join(HERE, 'jemu', JEMU_LINUX_DIR)
elif sys.platform.startswith(CORE_MAC_OS):
    JEMU_DIR = os.path.join(HERE, 'jemu', JEMU_MAC_DIR)
elif sys.platform.startswith(CORE_WINDOWS_OS1) or sys.platform.startswith(CORE_WINDOWS_OS2):
    JEMU_DIR = os.path.join(HERE, 'jemu', JEMU_WINDOWS_DIR)


def get_jemu_path():
    jemu_exe = 'jemu.exe' if sys.platform.startswith(CORE_WINDOWS_OS1) or sys.platform.startswith(CORE_WINDOWS_OS2) \
        else 'jemu'
    return os.path.join(JEMU_DIR, jemu_exe)



"""
:copyright: (c) 2017 by Jumper Labs Ltd.
:license: Apache 2.0, see LICENSE.txt for more details.
"""
from jemu_mem_peripheral import JemuMemPeripheral


class JemuBQ24160(JemuMemPeripheral):
    _COMMAND = "command"
    _INTERRUPT_TYPE = "interrupt_type"
    _COMMAND_BATTERY_USB_CHARGE = "battery_charge_usb_event"
    _COMMAND_BATTERY_IN_CHARGE = "battery_charge_in_event"
    _COMMAND_BATTERY_DISCHARGE = "battery_discharge_event"
    _COMMAND_SET_INTERRUPT = "set_interrupt"

    # interrupt types
    TIMER_FAULT_INTERRUPT = "timer_fault_interrupt"
    WATCHDOG_EXPIRATION_INTERRUPT = "watchdog_expiration_interrupt"
    SLEEP_MODE_INTERRUPT = "sleep_mode_interrupt"
    TEMPERATURE_FAULT_INTERRUPT_ = "temperature_fault_interrupt"
    BATTERY_FAULT_INTERRUPT = "battery_fault_interrupt"
    THERMAL_SHUTDOWN_INTERRUPT = "thermal_shutdown_interrupt"
    IN_SUPPLY_INTERRUPT = "in_supply_interrupt"
    USB_SUPPLY_INTERRUPT = "usb_supply_interrupt"

    def __init__(self, jemu_connection, id, peripheral_type, generators):
        JemuMemPeripheral.__init__(self, jemu_connection, id, peripheral_type, generators)
        
    def _charge_json(self, charge_type):
        return {
            self._COMMAND: charge_type,
        }

    def _no_charge_json(self):
        return {
            self._COMMAND: self._COMMAND_BATTERY_DISCHARGE,
        }

    def _battery_interrupt_json(self, interrupt):
        return {
            self._COMMAND: self._COMMAND_SET_INTERRUPT,
            self._INTERRUPT_TYPE: interrupt,
        }

    def charge(self, charge_type):
        """
          :param charge_type: usb ot in
        This function informs the peripheral to start charging.
        During charging, stat and int pins sets to low.
        """
        if charge_type == "usb":
            self._jemu_connection.send_command_async(self._charge_json(self._COMMAND_BATTERY_USB_CHARGE), self._id, self._peripheral_type)
        elif charge_type == "in":
            self._jemu_connection.send_command_async(self._charge_json(self._COMMAND_BATTERY_IN_CHARGE), self._id, self._peripheral_type)
        else:
            raise Exception("BQ24160: charge expect to get 'usb' or 'in' value")

    def charge_completed(self):
        """
        This function informs the peripheral when charging is complete.
        On charging complete, stat and int pins sets to high impedance.
        """
        self._jemu_connection.send_command_async(self._no_charge_json(), self._id, self._peripheral_type)

    def discharge(self):
        """
        This function disables charging operation.
        On charging disable, stat and int pins sets to high impedance.
        """
        self._jemu_connection.send_command_async(self._no_charge_json(), self._id, self._peripheral_type)

    def interrupt(self, interrupt):
        """
        This function activates an interrupt in the bq24160 peripheral.
        On interrupt a 128 microsecond pulse is sent out over stat and int pins.
        :param interrupt: Interrupt type
        """
        self._jemu_connection.send_command_async(self._battery_interrupt_json(interrupt), self._id, self._peripheral_type)


"""
:license: Apache 2.0, see LICENSE.txt for more details.
:license: Apache 2.0, see LICENSE for more details.
"""
import os
import sys
import json
from setuptools import setup, find_packages
from setuptools.command.install import install
from setuptools.command.develop import develop

here = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(here, 'jumper', '__version__.py'), 'r') as f:
    exec(f.read(), about)
print(about)
version = about['__version__']

files = ['LICENSE.txt', 'NOTICE.txt', 'hci/bsp.json', 'hci/program.bin', 'hci/instructions.so', 'stm32f4_default_board.json', 'nrf52_default_board.json', 'examples_hash_list.json']
if sys.platform.startswith('linux'):
    files.append('jemu/jemu-linux/jemu')
elif sys.platform.startswith('darwin'):
    files.append('jemu/jemu-mac/jemu')
elif sys.platform.startswith('win') or sys.platform.startswith('cygwin'):
    files.append('jemu/jemu-windows/jemu.exe')
    files.append('jemu/jemu-windows/libwinpthread-1.dll')


def send_install_event():
    config_file_name = 'config.json'
    JUMPER_DIR = os.path.join(os.path.expanduser('~'), '.jumper')
    config_file = os.path.join(JUMPER_DIR, config_file_name)

    if not os.path.isfile(config_file):
        return
    with open(config_file) as config_data:
        config = json.load(config_data)
    if 'token' in config:
        secret_token = config['token']

    from jumper.jemu_web_api import JemuWebApi
    web_api = JemuWebApi(jumper_token=secret_token)
    try:
        web_api.send_event({'event': 'SDK Installed'})
    except:
        print("error sending sdk installed event")


class PostDevelopCommand(develop):
    """Post-installation for development mode."""
    def run(self):
        develop.run(self)
        send_install_event()


class PostInstallCommand(install):
    """Post-installation for installation mode."""
    def run(self):
        install.run(self)
        send_install_event()


setup(
    name='jumper',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=version,

    description="SDK for using Jumper's emulator",
    # long_description=long_description,

    # The project's main homepage.
    url='https://vlab.jumper.io',

    # Author details
    author='Jumper Labs Ltd.',
    author_email='info@jumper.io',

    # Choose your license
    license='Apache 2.0',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        # 'Intended Audience :: Developers',
        # 'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        # 'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        # 'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
    ],

    # What does your project relate to?
    keywords='',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    # packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    packages=['jumper'],
    package_dir={'jumper': 'jumper'},

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['future', 'timeout-decorator', 'requests', 'termcolor', 'terminaltables'],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        'dev': ['sphinx', 'nose', 'sphinx-bootstrap-theme', 'sphinx_rtd_theme', 'twine', 'wheel']
    },

    # project only runs on certain Python versions (2.7)
    python_requires='==2.7.*',

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    # package_data={
    #     'sample': ['package_data.dat'],
    # },
    package_data={'jumper': files},
    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    # data_files=[('my_data', ['data/data_file'])],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        'console_scripts': [
            'jumper = jumper.__main__:main',
        ]
    },

    cmdclass={
        'develop': PostDevelopCommand,
        'install': PostInstallCommand,
    }
)



"""
:copyright: (c) 2017 by Jumper Labs Ltd.
:license: Apache 2.0, see LICENSE.txt for more details.
"""
from jemu_mem_peripheral import JemuMemPeripheral


class JemuBQ27421(JemuMemPeripheral):
    _INTERRUPT = "interrupts"
    _COMMAND_SET_INTERRUPT = "set_interrupt"
    _COMMAND = "command"

    def __init__(self, jemu_connection, id, peripheral_type, generators):
        JemuMemPeripheral.__init__(self, jemu_connection, id, peripheral_type, generators)

    def _battery_interrupt_json(self):
        return {
            self._COMMAND: self._COMMAND_SET_INTERRUPT,
        }

    def interrupt(self):
        """
        This function activates an interrupt in the bq27421 peripheral.
        On interrupt a 1 ms pulse is sent out over gpout pin.
        """
        self._jemu_connection.send_command_async(self._battery_interrupt_json(), self._id, self._peripheral_type)



"""
:copyright: (c) 2017 by Jumper Labs Ltd.
:license: Apache 2.0, see LICENSE.txt for more details.
"""
from threading import Lock
from common import JemuConnectionException


class JemuGpio:
    _PIN_NUM = "pin_number"
    _TRANSITION_TYPE = "transition_type"
    _DESCRIPTION = "description"
    _PIN_LEVEL_EVENT = "pin_level_event"
    _PIN_LEVEL = "pin_level"
    _PIN_NUMBER = "pin_number"
    _GET_PIN_LEVEL = "get_pin_level"
    _SET_PIN_LEVEL = "set_pin_level"
    _VALUE_RESPONSE = "value_response"
    _COMMAND = "command"
    _VALUE = "value"
    _ID = 12

    def __init__(self, mcu_name="nrf52832"):
        self._pin_level_callback = None
        self._jemu_socket_manager = None
        self._lock = Lock()
        self._peripheral_type = mcu_name

    def on_pin_level_event(self, callback):
        with self._lock:
            self._pin_level_callback = callback

    def receive_packet(self, jemu_packet):
        if jemu_packet[self._DESCRIPTION] == self._PIN_LEVEL_EVENT:
            with self._lock:
                if self._pin_level_callback:
                    self._pin_level_callback(jemu_packet[self._PIN_NUM], jemu_packet[self._TRANSITION_TYPE])

    def set_connection_manager(self, jemu_socket_manager):
        self._jemu_socket_manager = jemu_socket_manager
        self._jemu_socket_manager.register(self.receive_packet)

    def _get_pin_level_message(self, pin_num):
        return {
            self._COMMAND: self._GET_PIN_LEVEL,
            self._PIN_NUMBER: pin_num
        }

    def _set_pin_level_message(self, pin_num, pin_level):
        return {
            self._COMMAND: self._SET_PIN_LEVEL,
            self._PIN_LEVEL: pin_level,
            self._PIN_NUMBER: pin_num
        }

    def get_pin_level(self, pin_num):
        jemu_packet = self._jemu_socket_manager.send_command(self._get_pin_level_message(pin_num), self._ID, self._peripheral_type)
        if self._DESCRIPTION in jemu_packet and jemu_packet[self._DESCRIPTION] == self._VALUE_RESPONSE and self._VALUE in jemu_packet:
            return jemu_packet[self._VALUE]
        else:
            raise JemuConnectionException("Error: Couldn't get pin [{}] level".format(pin_num))

    def set_pin_level(self, pin_num, pin_level):
        self._jemu_socket_manager.send_command_async(self._set_pin_level_message(pin_num, pin_level), self._ID, self._peripheral_type)



from jemu_mem_peripheral import JemuMemPeripheral


class JemuBMA280(JemuMemPeripheral):
    _VALUE = "value"
    _INTERRUPT = "interrupts"
    _COMMAND_SET_INTERRUPT = "set_interrupt"
    _COMMAND_UNSET_INTERRUPT = "unset_interrupt"
    _COMMAND_RESET_INTERRUPT = "reset_interrupts"
    _COMMAND = "command"


    class Interrupts(object):
        SLOPE_X = "slope_x"
        SLOPE_Y = "slope_y"
        SLOPE_Z = "slope_z"
        LOW_G = "low_g"
        HIGH_G_X = "high_g_x"
        HIGH_G_Y = "high_g_y"
        HIGH_G_Z = "high_g_z"
        DOUBLE_TAP_X = "double_tap_x"
        DOUBLE_TAP_Y = "double_tap_y"
        DOUBLE_TAP_Z = "double_tap_z"
        SINGLE_TAP_X = "single_tap_x"
        SINGLE_TAP_Y = "single_tap_y"
        SINGLE_TAP_Z = "single_tap_z"
        ORIENT_XY = "orient_xy"
        ORIENT_Z = "orient_z"
        FLAT = "flat"
        FIFO_FULL = "fifo_full"
        FIFO_WATERMARK = "fifo_watermark"
        NEW_DATA = "new_data"

    def __init__(self, jemu_connection, id, peripheral_type, generators):
        JemuMemPeripheral.__init__(self, jemu_connection, id, peripheral_type, generators)

    def _set_interrupt_json(self, interrupt):
        return {
            self._INTERRUPT: interrupt,
            self._COMMAND: self._COMMAND_SET_INTERRUPT,
        }

    def _unset_interrupt_json(self, interrupt):
        return {
            self._COMMAND: self._COMMAND_UNSET_INTERRUPT,
            self._INTERRUPT: interrupt,
        }

    def _reset_interrupt_json(self, interrupt):
        return {
            self._COMMAND: self._COMMAND_RESET_INTERRUPT,
            self._INTERRUPT: interrupt,
        }

    def set_interrupt(self, interrupt):
        self._jemu_connection.send_command_async(self._set_interrupt_json(interrupt), self._id, self._peripheral_type)

    def unset_interrupt(self, interrupt):
        self._jemu_connection.send_command_async(self._unset_interrupt_json(interrupt), self._id, self._peripheral_type)

    def reset_interrupts(self):
        self._jemu_connection.send_command_async(self._reset_interrupt_json('reset_interrupt'), self._id, self._peripheral_type)



"""
:copyright: (c) 2017 by Jumper Labs Ltd.
:license: Apache 2.0, see LICENSE.txt for more details.
"""
from __future__ import print_function
import argparse
import signal
from os import getcwd
import os
from time import sleep
from threading import Event
import signal
import sys
import re

from __version__ import __version__ as jumper_current_version
from .vlab import Vlab
from .vlab_hci_device import VirtualHciDevice
from . import __version__
from .common import VlabException

stop = Event()


def signal_handler(signum, frame):
    stop.set()


signal.signal(signal.SIGTERM, signal_handler)
signal.signal(signal.SIGINT, signal_handler)
signal.signal(signal.SIGABRT, signal_handler)


class _VersionAction(argparse.Action):
    def __init__(self,
                 option_strings,
                 version=None,
                 dest=argparse.SUPPRESS,
                 default=argparse.SUPPRESS,
                 help="show program's version number and exit"):
        super(_VersionAction, self).__init__(
            option_strings=option_strings,
            dest=dest,
            default=default,
            nargs=0,
            help=help)
        self.version = version

    def __call__(self, parser, namespace, values, option_string=None):
        version = self.version
        if version is None:
            version = parser.version
        formatter = parser._get_formatter()
        formatter.add_text(version)
        Vlab.check_sdk_version(False)
        parser.exit(message=formatter.format_help())


def validate_traces_list(traces_list):
    traces_list = traces_list.split(',')
    for trace in traces_list:
        if trace not in ['regs', 'interrupts', 'functions']:
            raise VlabException('jumper run: error: unrecognized trace:' + trace, 1)


def run(args):
    registers_trace = False
    functions_trace = False
    interrupts_trace = False

    if args.traces_list:
        for trace in args.traces_list.split(','):
            if trace == 'registers' or trace == 'regs':
                registers_trace = True
            elif trace == 'functions':
                functions_trace = True
            elif trace == 'interrupts':
                interrupts_trace = True
            else:
                raise VlabException('jumper run: error: Invalid trace type {}. Valid traces are regs, functions, interrupts'.format(trace), 1)

    vlab = None
    try:
        vlab = Vlab(
            working_directory=args.working_directory,
            sudo_mode=args.sudo_mode,
            gdb_mode=args.gdb_mode,
            registers_trace=registers_trace,
            functions_trace=functions_trace,
            interrupts_trace=interrupts_trace,
            trace_output_file=args.trace_output_file,
            print_uart=args.print_uart,
            uart_output_file=args.uart_output_file,
            uart_device=args.uart_device,
            platform=args.platform,
            token=args.token,
            debug_peripheral=args.debug_peripheral,
            gpio=args.gpio
        )
        vlab.load(args.fw)

        reached_bkpt = Event()

        def bkpt_callback(code):
            print('Firmware reached a BKPT instruction with code {}'.format(code))
            reached_bkpt.set()

        vlab.on_bkpt(bkpt_callback)
        ns = None

        if args.stop_after:
            vlab._send_stop_after_event()

            def raise_invalid_time():
                print('Invalid time format for --stop-after: {}'.format(args.stop_after))
                return 1

            match = re.match('^\d+', args.stop_after)
            if not match:
                raise_invalid_time()
            ns = int(match.group(0))

            if re.match('^\d+$', args.stop_after) or re.match('^\d+ms$', args.stop_after):
                ns = ns * 1000 * 1000
            elif re.match('^\d+s$', args.stop_after):
                ns = ns * 1000 * 1000 * 1000
            elif re.match('^\d+us$', args.stop_after):
                ns = ns * 1000
            elif re.match('^\d+ns$', args.stop_after):
                pass
            else:
                raise_invalid_time()

        if not args.print_uart and not args.traces_list:
            print(
                '\nVirtual device is running without UART/Trace prints (use -u and/or -t to get your firmware '
                'execution status)\n')
        else:
            print('\nVirtual device is running\n')
            sys.stdout.flush()

        vlab.start(ns)

        try:
            while (not reached_bkpt.is_set()) and not stop.is_set() and vlab.get_state() == 'running':
                sleep(0.5)
        except KeyboardInterrupt:
            pass

        vlab.stop()
        return vlab.get_return_code()

    except VlabException as e:
        print('jumper run: error: {}'.format(e.message))
        try:
            if vlab:
                vlab.stop()
        finally:
            return e.exit_code


def hci():
    Vlab.check_sdk_version(False)
    virtual_hci_device = VirtualHciDevice()
    
    virtual_hci_device.start()
    while not stop.wait(timeout=0.1):
        pass

    virtual_hci_device.stop()


def command_run(args):
    if args.version:
        print("v" + jumper_current_version)
        return 0

    if not args.fw:
        print("jumper run: error: argument --bin/-b is required")
        return 1

    if args.traces_list:
        args.traces_list = args.traces_list.lower().replace(' ', '')
        validate_traces_list(args.traces_list)

    return run(args)


def main():
    parser = argparse.ArgumentParser(
        prog='jumper',
        description="CLI interface for using Jumper's emulator"
    )

    parser.add_argument('--version', action=_VersionAction, version='%(prog)s {}'.format(__version__))
    parser.add_argument('--token', default=None, dest='token', type=str, help='Your secret token')

    subparsers = parser.add_subparsers(title='Commands', dest='command')

    run_parser = subparsers.add_parser(
        'run',
        help='Executes a FW file on a virtual device. Currently only support nRF52 devices'
    )
    run_parser.add_argument(
        '--fw',
        '--bin',
        '-b ',
        help="Firmware to be flashed to the virtual device (supported extensions are bin, out, elf, hex). In case more than one file needs to be flashed (such as using Nordic's softdevice), the files should be merged first. Check out https://vlab.jumper.io/docs#softdevice for more details",
    )

    run_parser.add_argument(
        '--debug-peripheral',
        action='store_true',
        help="Debug peripherals, enables to attach debugger to pid",
        default=False,
        dest='debug_peripheral'
    )

    run_parser.add_argument(
        '--directory',
        '-d ',
        help='Working directory, should include the board.json and scenario.json files. Default is current working directory',
        default=getcwd(),
        dest='working_directory'
    )

    run_parser.add_argument(
        '--sudo',
        '-s ',
        help='Run in sudo mode => FW can write to read-only registers. This should usually be used for testing low-level drivers, fuzzing (error injection) and certification tests.',
        action='store_true',
        default=False,
        dest='sudo_mode'
    )

    run_parser.add_argument(
        '--gdb',
        '-g ',
        help='Opens a GDB port for debugging the FW on port 5555. The FW will not start running until the GDB client connects.',
        action='store_true',
        default=False,
        dest='gdb_mode'
    )

    run_parser.add_argument(
        '--version',
        '-v ',
        help='Jumper sdk version.',
        action='store_true',
        default=False
    )

    run_parser.add_argument(
        '--trace',
        '-t ',
        help=
        """
        Prints a trace report to stdout.
        Valid reports: regs,interrupts,functions. (the functions trace can only be used with an out/elf file) 
        Example: jumper run --fw my_bin.bin -t interrupts,regs --trace-dest trace.txt
        Default value: regs 
        This can be used with --trace-dest to forward it to a file.
        """,
        const='regs',   # default when there are 0 arguments
        nargs='?',      # 0-or-1 arguments
        dest='traces_list'
    )

    run_parser.add_argument(
        '--trace-dest',
        type=str,
        help=
        """
        Forwards the trace report to a destination file. Must be used with -t.
        To print to stdout, just hit -t.
        """,
        default='',
        dest='trace_output_file'
    )

    run_parser.add_argument(
        '--uart',
        '-u ',
        action='store_true',
        default=False,
        help='Forwards UART prints to stdout, this can be used with --uart-dest to forward it to a file.',
        dest='print_uart'
    )

    run_parser.add_argument(
        '--uart-dest',
        type=str,
        help=
        """
        Forwards UART prints to a destination file. This MUST be used -u with this flag to make it work.
        To print to stdout, just hit -u.
        """,
        default=None,
        dest='uart_output_file'
    )

    run_parser.add_argument(
        '--uart-device',
        action='store_true',
        default=False,
        help='Creates a "uart" file in the working directory which is linked to the virtual UART device. Can be used with "screen". Linux only',
        dest='uart_device'
    )

    run_parser.add_argument(
        "--gpio",
        help=
        """
        Prints GPIO events (pin changes) to stdout.
        """,
        action='store_true',
        default=False,
    )

    run_parser.add_argument(
        '--platform',
        '-p ',
        type=str,
        choices=['nrf52832', 'stm32f4'],
        help=
        """
        Sets platform type. (Valid platforms: nrf52832,stm32f4).
        """,
        default=None,
        dest='platform'
    )

    run_parser.add_argument(
        '--stop-after',
        type=str,
        help=
        """
        Stop the execution after a specific amount of time. Units should be stated, if no units are stated, time in ms is assumed.
        Examples: "--stop-after 1s, --stop-after 1000ms, --stop-after 1000000us, --stop-after 1000000000ns" 
        """,
        default=None,
        dest='stop_after'
    )

    subparsers.add_parser(
        'ble',
        help='Creates a virtual HCI device (BLE dongle) for regular Linux/Bluez programs to communicate with virtual devices'
    )

    args = parser.parse_args()

    if args.command == 'run':
        return command_run(args)

    if args.command == 'ble':
        hci()
        return 0


if __name__ == '__main__':
    try:
        exit(main())
    except VlabException as e:
        print(e.message)
        exit(e.exit_code())



from __future__ import print_function
from threading import Thread
import functools
from .common import TimeoutException


def timeout(timeout):
    def deco(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            res = [TimeoutException('function [%s] timeout [%s seconds] exceeded!' % (func.__name__, timeout))]

            def newFunc():
                try:
                    res[0] = func(*args, **kwargs)
                except Exception as e:
                    res[0] = e
            t = Thread(target=newFunc)
            t.daemon = True
            try:
                t.start()
                t.join(timeout)
            except Exception as je:
                print('error starting thread')
                raise je
            ret = res[0]
            if isinstance(ret, BaseException):
                raise ret
            return ret
        return wrapper
    return deco


try:
    result = timeout(timeout=16)(MyModule.MyFunc)(MyArgs)
except:
    pass
